// This code borrowed (and modified) from Chuck Baker and FEBE 6.0. Thanks!
// Drag and drop code taken from template at
// http://kb.mozillazine.org/Dev_:_Extensions_:_Example_Code_:_Adding_Drag_and_Drop_to_Statusbarpanel

var SimpleClocksStatObserver = {
    init: function () {
        // Any text you want but should have something to do with the extension name.
        SimpleClocksStatObserver.flavour = "SimpleClocksLocationsFla";
        // id of the statusbarpanel to drag.
        SimpleClocksStatObserver.statusbarPanel = "simclk-statpanelTimer";
        // id of statusbar.
        SimpleClocksStatObserver.statbar = "status-bar";
        SimpleClocksStatObserver.prefs = Components.classes["@mozilla.org/preferences-service;1"].
                getService(Components.interfaces.nsIPrefService).
                getBranch("extensions.simpletimerClocks@grbradt.org.");

        // Checks whether the preference has been changed from the default value (empty string)
        // and inserts our panel before the element referenced in preference.
        // Default position is last.
        if ( SimpleClocksStatObserver.prefs.prefHasUserValue("insertBefore") &&
             SimpleClocksStatObserver.prefs.getCharPref("insertBefore") !== "last" ) {
            document.getElementById(SimpleClocksStatObserver.statbar).
                    insertBefore(document.getElementById(SimpleClocksStatObserver.statusbarPanel),
                                 document.getElementById(SimpleClocksStatObserver.prefs.getCharPref("insertBefore")));
        }
        else {
            document.getElementById(SimpleClocksStatObserver.statbar).
                    appendChild(document.getElementById(SimpleClocksStatObserver.statusbarPanel));
        }
    },

    debug: function (aMsg) {
        setTimeout(function() { throw new Error("[debug] " + aMsg); }, 0);
    },

    // The type of data being dragged is stored as a set of flavours.
    // Often, a dragged object will be available in a number of flavours.
    // That way, a drop target can accept the flavour it finds most suitable.
    getSupportedFlavours: function () {
        var flavours = new FlavourSet();
        flavours.appendFlavour(SimpleClocksStatObserver.flavour); // text/unicode
        return flavours;
    },

    onDragStart: function (evt, transferData, action) {
        var elme = evt.target;

        // See if target element was child label or image, eg.
        while ( elme != document.getElementById(SimpleClocksStatObserver.statusbarPanel) ) {
            elme = elme.parentNode;
        }

        // Data being transferred is our panel id, in text/unicode format
        var txt = elme.getAttribute("id");
        transferData.data = new TransferData();
        transferData.data.addDataForFlavour(SimpleClocksStatObserver.flavour, txt);
        var statusPanel = document.getElementById(SimpleClocksStatObserver.statusbarPanel);
        var statusbar = document.getElementById(SimpleClocksStatObserver.statbar);
        var child = statusbar.firstChild;
        var x = 0;

        // Check if each panel has id, add listeners to each panel.
        // This code assumes all children of statusbar are statusbarpanels,
        // which is not necessarily true!
        while ( child ) {
            if ( child != statusPanel ) {
                if ( !child.id ) {
                    // If you insert before a no-id, positioning is lost
                    // on browser restart. id doesn't persist.
                    var newId = "statusbarpanel-noID" + x;
                    while ( document.getElementById(newId) ) {
                        newId += "x" + x;
                    }
                    child.id = newId;
                    child.setAttribute("persist",
                                       new String("id" + (child.persist ? " " + child.persist : "")));
                    x++;
                }

                if ( child.childNodes.length == 0 &&
                     document.getAnonymousNodes(child).length == 0 ) {
                    child.setAttribute("abnochildren", true);
                }

                child.addEventListener("dragenter", function(event) { nsDragAndDrop.dragEnter(event, SimpleClocksStatObserver); }, false);
                child.addEventListener("dragover", function(event) { nsDragAndDrop.dragOver(event, SimpleClocksStatObserver); }, false);
                child.addEventListener("dragdrop", function(event) { nsDragAndDrop.drop(event, SimpleClocksStatObserver); }, false);
            }

            child = child.nextSibling;
        }

        window.addEventListener("dragexit", function(event) { nsDragAndDrop.dragExit(event, SimpleClocksStatObserver); }, true);
    },

    onDragEnter: function (evt, flavour, session) {
        var elm = evt.target;

        while( elm.nodeName != "statusbarpanel" ) {
            elm = elm.parentNode;
        }
    },

    onDragOver: function (evt, flavour, session) {
        var elm = evt.target;

        while( elm.parentNode.nodeName != "statusbar" ) {
           elm = elm.parentNode;
        }
    },

    onDrop: function (evt, dropdata, session) {
        if ( dropdata.data != "" ) {
            var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                    getService(Components.interfaces.nsIPrefService).
                    getBranch("extensions.simpletimerClocks@grbradt.org.");
            var elm = evt.target;

            // May have dropped on image or label.
            while( elm.nodeName != "statusbarpanel" ) {
                elm = elm.parentNode;
            }

            // 'delta' is the difference between the mouse position when
            // the drop event occurs and the middle of the target's boxObject
            // delta > 0 if the mouse is on the right of the middle
            var delta = evt.clientX - (elm.boxObject.x + elm.boxObject.width / 2);
            var elem = document.getElementById(dropdata.data);
            var parent = elem.parentNode;

            if ( delta > 0 ) {
                elm = elm.nextSibling;
                if ( elm == elem ) {
                    elm = elm.nextSibling;
                }
            }

            // Don't insert before non-panel statusbar children eg popupset.
            while( elm && (elm.nodeName != "statusbarpanel" || elm == elem) ) {
                elm = elm.nextSibling;
            }

            if ( elm ) {
                parent.insertBefore(elem, elm);
                elem.setAttribute("insertbefore", "" + elm.getAttribute("id"));
                prefs.setCharPref("insertBefore", elm.getAttribute("id") + "");
            }
            else {
                parent.appendChild(elem);
                elem.setAttribute("insertbefore", "last");
                prefs.setCharPref("insertBefore", "last");
            }
        }
    }
};
window.addEventListener('load', SimpleClocksStatObserver.init, false);
